<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns="urn:hl7-org:v3" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0" xmlns:hl7="urn:hl7-org:v3" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xsl:output method="xml" indent="yes" exclude-result-prefixes="#all"/>
    <xsl:strip-space elements="*"/>
    <!-- Transforms MP 9.03 message/CDA into MP 9.04 -->

    <!-- TemplateId 9119 bij patiënt mág niet meer voorkomen, zie versie log Alexander Henket -->
    <xsl:template match="hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9119']">
        <!-- Do nothing -->
    </xsl:template>

    <!-- Issue 238 , patiënt als auteur in author, bij medicatiegebruik en voorstel ma en voorstel vv -->
    <xsl:template match="hl7:substanceAdministration[hl7:participant[@typeCode = 'AUT']/hl7:participantRole/@classCode = 'PAT']/hl7:consumable
        | hl7:supply[hl7:participant[@typeCode = 'AUT']/hl7:participantRole/@classCode = 'PAT']/hl7:product">
        <xsl:variable name="performer" select="../hl7:performer"/>
        <!-- author komt na (optionele) performer na verplichte consumable/product, eerst consumable/product kopiëren-->
        <xsl:copy-of select="." exclude-result-prefixes="#all"/>
        <!-- dan een eventuele performer kopiëren -->
        <xsl:copy-of select="$performer" exclude-result-prefixes="#all"/>
        <!-- dan author toevoegen -->
        <author xsl:exclude-result-prefixes="#all">
            <xsl:apply-templates select="../hl7:participant/hl7:time"/>
            <assignedAuthor xsl:exclude-result-prefixes="#all">
                <xsl:apply-templates select="/*/hl7:recordTarget/hl7:patientRole/hl7:id"/>
                <code code="ONESELF" displayName="Self" codeSystem="2.16.840.1.113883.5.111" codeSystemName="HL7 Role code"/>
            </assignedAuthor>
        </author>
    </xsl:template>
    <!-- Issue 238 vervolg, patiënt als auteur in author, de 'oude' participant author en reeds gekopieerde performer verwijderen -->
    <xsl:template match="
        hl7:substanceAdministration/hl7:participant[@typeCode = 'AUT'][hl7:participantRole/@classCode = 'PAT']
      | hl7:substanceAdministration[hl7:participant[@typeCode = 'AUT']/hl7:participantRole/@classCode = 'PAT']/hl7:performer 
                       | hl7:supply[hl7:participant[@typeCode = 'AUT']/hl7:participantRole/@classCode = 'PAT']/hl7:performer
                       | hl7:supply/hl7:participant[@typeCode = 'AUT'][hl7:participantRole/@classCode = 'PAT']">
        <!-- Niet meer opnemen -->
    </xsl:template>

    <!-- Issue 245. Relatie naar medicamenteuze behandeling móét worden opgenomen, indien onbekend: met id/@nullFlavor -->
    <!-- supply: verstrekking / verstrekkingsverzoek -->
    <!-- substanceAdministration: medicatieafspraak / toedieningsafspraak, maar niet verplicht voor medicatiegebruik -->
    <xsl:template match="
            hl7:supply[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9094'][not(hl7:entryRelationship/*/hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9084'])]
            | hl7:supply[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9087'][not(hl7:entryRelationship/*/hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9084'])]
            | hl7:substanceAdministration[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9152'][not(hl7:entryRelationship/*/hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9084'])]
            | hl7:substanceAdministration[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9148'][not(hl7:entryRelationship/*/hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9084'])]">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
            <!--Relatie naar medicamenteuze behandeling, als laatste in de bouwsteen -->
            <entryRelationship typeCode="COMP" inversionInd="true">
                <procedure classCode="PROC" moodCode="EVN">
                    <templateId root="2.16.840.1.113883.2.4.3.11.60.20.77.10.9084"/>
                    <id nullFlavor="NI"/>
                    <code code="1" displayName="Medicamenteuze behandeling" codeSystem="2.16.840.1.113883.2.4.3.11.60.20.77.5.3"/>
                </procedure>
            </entryRelationship>
        </xsl:copy>
    </xsl:template>

    <!-- Issue 246, 247, 248, 249, 254, 255. Geen wijzigingen in instance nodig -->

    <!-- Issue 256. Bij verstrekkingsverzoek en bij voorstel verstrekkingsverzoek ontbrak code element -->
    <xsl:template match="
        hl7:supply[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9087'][not(hl7:code)]/hl7:id
        |  hl7:supply[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9131'][not(hl7:code)]/hl7:id">
        <xsl:copy-of select="." exclude-result-prefixes="#all"/>
        <code code="3" codeSystem="2.16.840.1.113883.2.4.3.11.60.20.77.5.3" displayName="Verstrekkingsverzoek"/>
    </xsl:template>

    <!-- Issue 257. Geen wijzigingen in instance nodig -->

    <!-- Issue 259. Onterecht een low/high bij expectedUseTime in verstrekking, gewoon verwijderen -->
    <xsl:template match="
            hl7:supply[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9094']/hl7:expectedUseTime/hl7:low
            | hl7:supply[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9094']/hl7:expectedUseTime/hl7:high">
        <!-- Do nothing -->
    </xsl:template>

    <!-- Issue 260. Geen wijzigingen in instance nodig -->

    <!-- Issue 261. entryRelationship moet typeCode RSON hebben bij redenen ma (9114 en 9115) -->
    <xsl:template match="
            hl7:entryRelationship[*/hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9114'][not(@typeCode = 'RSON')]
            | hl7:entryRelationship[*/hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9115'][not(@typeCode = 'RSON')]">
        <xsl:copy>
            <xsl:attribute name="typeCode" select="'RSON'"/>
            <xsl:apply-templates select="node()"/>
        </xsl:copy>
    </xsl:template>

    <!-- Issue 263. Geen wijzigingen in instance nodig -->
    <!-- Issue 266. informant=zorgverlener medicatiegebruik moet ander template gebruiken. Geen wijzigingen in instance nodig -->

    <!-- Issue 267, 269, 270. Geen wijzigingen in instance nodig -->

    <!-- Issue 271 TemplateId opnemen is optioneel, weghalen altijd goed, laten staan soms fout omdat nu ander template gebruikt wordt -->
    <xsl:template match="
            hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9066']
            | hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9113']
            | hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9167']">
        <!-- Do nothing -->
    </xsl:template>

    <!--  Issue 272. Toelichting in voorstel transacties toegevoegd. Geen wijzigingen in instance nodig -->
    <!-- Issue 273, 274, 275. Geen wijzigingen in instance nodig -->

    <!-- Issue 276.  Bij aanvullende informatie was onterecht een datatype 'CD' gebruikt. Dit is hersteld naar 'CE'-->
    <xsl:template match="hl7:value[@xsi:type = 'CD']">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:attribute name="xsi:type" select="'CE'"/>
        </xsl:copy>
    </xsl:template>

    <!-- Issue 277. Ander template voor MA in een voorstel vv en ander template voor consumable in die MA -->
    <!-- ander template id voor MA -->
    <xsl:template match="
            hl7:organizer[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9130']//hl7:substanceAdministration/hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9148']
            | hl7:ClinicalDocument[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9144']//hl7:substanceAdministration/hl7:templateId[@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9148']">
        <xsl:copy>
            <xsl:attribute name="root" select="'2.16.840.1.113883.2.4.3.11.60.20.77.10.9185'"/>
        </xsl:copy>
    </xsl:template>
    <!-- Issue 277 vervolg.  ander template voor consumable in die MA-->
    <xsl:template match="
            hl7:organizer[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9130']//hl7:substanceAdministration[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9148']/hl7:consumable/hl7:manufacturedProduct/hl7:templateId
            | hl7:ClinicalDocument[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9144']//hl7:substanceAdministration[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9148']/hl7:consumable/hl7:manufacturedProduct/hl7:templateId">
        <xsl:copy>
            <xsl:attribute name="root" select="'2.16.840.1.113883.2.4.3.11.60.20.77.10.9163'"/>
        </xsl:copy>
    </xsl:template>

    <!-- Issue 282, 283. Updates aan xml schemas.  Geen wijzigingen in instance nodig -->

    <!-- Issue 285. Medicatiegebruik. Patiënt als informant nu in xml element informant -->
    <xsl:template match="hl7:substanceAdministration[hl7:templateId/@root = '2.16.840.1.113883.2.4.3.11.60.20.77.10.9154']/hl7:participant[@typeCode = 'INF'][hl7:participantRole/@classCode = 'PAT']">
        <informant xsl:exclude-result-prefixes="#all">
            <assignedEntity>
                <xsl:apply-templates select="/*/hl7:recordTarget/hl7:patientRole/hl7:id"/>
                <code code="ONESELF" displayName="Self" codeSystem="2.16.840.1.113883.5.111" codeSystemName="HL7 Role code"/>
            </assignedEntity>
        </informant>
    </xsl:template>

    <!-- Default copy xsl template -->
    <xsl:template match="@* | node()">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>
